@@ -6,13 +6,11 @@
 #' `fish` object, when created, spawns a detached Stockfish process and pipes
 #' into its stdin and stdout.
 #'
-#' For more information, see the full documentation by running `?fish`.
-#'
 #' @section Bundled Stockfish:
 #' This package comes bundled with
 #' [Stockfish](https://github.com/official-stockfish/Stockfish), a very popular,
 #' open source, powerful chess engine written in C++. It can achieve an ELO of
-#' 3516, runs on Windows, macOS, Linux, iOS and Android, and can be compiled in
+#' 3544, runs on Windows, macOS, Linux, iOS and Android, and can be compiled in
 #' less than a minute.
 #'
 #' When installing `{stockfish}` (lower case), Stockfish's (upper case) source
@@ -20,12 +18,19 @@
 #' This is not a system-wide installation! You don't have to give it
 #' administrative privileges to run or even worry about any additional software.
 #'
-#' The only downside is that the bundled version of the engine is Stockfish 11,
-#' not the most recent release. This is because, since version 12, the engine
-#' needs additional downloads, which would dramatically increase installation
-#' time. If you want to, you can [download](https://stockfishchess.org/download/)
-#' the version of your choosing and pass the executable as an argument to
-#' `fish$new()`.
+#' But there are two main downsides:
+#'
+#' * While the bundled version of the engine (Stockfish 14.1) is up-to-date as
+#'   of March 2022, it isn't able to update itself. This means that, if I'm not
+#'   able to port an upcomming version of Stockfish, the package will stay
+#'   behind. Luckly, you can always download the version of your choosing and
+#'   pass the executable as an argument to `new()`.
+#'
+#' * Since version 12, Stockfish supports NNUE evaluation but this requires
+#'   some pretty heavy binaries. In order to avoid bundling large files with
+#'   `{stockfish}`, **I have decided to disable NNUE** in the source code.
+#'   Again, you are free to download a NNUE-capable version and use it instead
+#'   of the bundled executable.
 #'
 #' @section UCI Protocol:
 #' UCI (Universal Chess Interface) is an open communication protocol that
@@ -67,21 +72,23 @@
 #' [UCI's documentation](http://wbec-ridderkerk.nl/html/UCIProtocol.html)), but
 #' here are a few things to look out for:
 #'
-#' - Not every UCI method is implemented: since `{stockfish}` was made with
-#' Stockfish (and mainly Stockfish 11) in mind, a couple of UCI methods that
-#' don't work with the engine were not implemented. They are `debug()` and
-#' `register()`.
-#' - Most methods return silently: since most UCI commands don't output anything
-#' or output boilerplate text, most methods of this class return silently. The
-#' exceptions are `run()`, `isready()`, `go()` and `stop()`; you can see exactly
-#' what they return by reading their documentations.
-#' - Not every Stockfish option will work: at least when using the bundled
-#' version of Stockfish, not every documented option will work with `setoption()`.
-#' This happens because, as described above, this package comes with Stockfish
-#' 11, which is not the most recent version. Options that will not work are
-#' labeled with an asterisk.
-#' - Times are in milliseconds: unlike most R functions, every method that takes
-#' a time interval expects them in milliseconds, not seconds.
+#' * Not every UCI method is implemented: since `{stockfish}` was made with
+#'   Stockfish in mind, a couple of UCI methods that don't work with the engine
+#'   were not implemented. They are `debug()` and `register()`.
+#'
+#' * Most methods return silently: since most UCI commands don't output anything
+#'   or output boilerplate text, most methods return silently. The
+#'   exceptions are `run()`, `isready()`, `go()` and `stop()`; you can see
+#'   exactly what they return by reading their documentations.
+#'
+#' * Not every Stockfish option will work: at least when using the bundled
+#'   version of Stockfish, not every documented option will work with
+#'   `setoption()`. This happens because, as described above, the bundled
+#'   version has some limitations. Options that will not work are labeled with
+#'   an asterisk.
+#'
+#' * Times are in milliseconds: unlike most R functions, every method that takes
+#'   a time interval expects them in milliseconds, not seconds.
 #'
 #' @export
 fish <- R6::R6Class(
@@ -113,7 +120,9 @@ fish <- R6::R6Class(
       exe <- if (is.null(path)) fish_find() else path.expand(path)
 
       # Start process
-      self$process <- processx::process$new(exe, stdin = "|", stdout = "|", stderr = "|")
+      self$process <- processx::process$new(
+        exe, stdin = "|", stdout = "|", stderr = "|"
+      )
 
       # Record output
       self$process$poll_io(100)
@@ -155,7 +164,7 @@ fish <- R6::R6Class(
         tmp <- self$process$read_output()
 
         if (tmp == "") {
-          break()
+          break
         }
 
         # Choose separator based on OS
@@ -205,72 +214,89 @@ fish <- R6::R6Class(
 
     #' @description Change the internal parameters of the engine. All currently
     #' supported options (according to Stockfish's documentation) are listed
-    #' below, but note that __those marked with an * require Stockfish 12__ to
-    #' work):
+    #' below, but note that **those marked with an * will not work** with the
+    #' buldled version:
     #'
-    #' - `Threads`: the number of CPU threads used for searching a position. For
-    #' best performance, set this equal to the number of CPU cores available.
-    #' - `Hash`: the size of the hash table in MB. It is recommended to set Hash
-    #' after setting Threads.
-    #' - `Ponder`: let Stockfish ponder its next move while the opponent is
-    #' thinking.
-    #' - `MultiPV`: output the N best lines (principal variations, PVs) when
-    #' searching. Leave at 1 for best performance.
-    #' - `Use NNUE`*: toggle between the NNUE and classical evaluation functions.
-    #' If set to "true", the network parameters must be available to load from
-    #' file (see also EvalFile), if they are not embedded in the binary.
-    #' - `EvalFile`*: the name of the file of the NNUE evaluation parameters.
-    #' Depending on the GUI the filename might have to include the full path to
-    #' the folder/directory that contains the file. Other locations, such as the
-    #' directory that contains the binary and the working directory, are also
-    #' searched.
-    #' - `UCI_AnalyseMode`: an option handled by your GUI.
-    #' - `UCI_Chess960`: an option handled by your GUI. If true, Stockfish will
-    #' play Chess960.
-    #' - `UCI_ShowWDL`*: tf enabled, show approximate WDL statistics as part of
-    #' the engine output. These WDL numbers model expected game outcomes for a
-    #' given evaluation and game ply for engine self-play at fishtest LTC
-    #' conditions (60+0.6s per game).
-    #' - `UCI_LimitStrength`*: enable weaker play aiming for an Elo rating as set
-    #' by UCI_Elo. This option overrides Skill Level.
-    #' - `UCI_Elo`*: if enabled by UCI_LimitStrength, aim for an engine strength
-    #' of the given Elo. This Elo rating has been calibrated at a time control of
-    #' 60s+0.6s and anchored to CCRL 40/4.
-    #' - `Skill Level`: lower the Skill Level in order to make Stockfish play
-    #' weaker (see also UCI_LimitStrength). Internally, MultiPV is enabled, and
-    #' with a certain probability depending on the Skill Level a weaker move
-    #' will be played.
-    #' - `SyzygyPath`: path to the folders/directories storing the Syzygy
-    #' tablebase files. Multiple directories are to be separated by ";" on
-    #' Windows and by ":" on Unix-based operating systems. Do not use spaces
-    #' around the ";" or ":". Example:
-    #' `C:\tablebases\wdl345;C:\tablebases\wdl6;D:\tablebases\dtz345;D:\tablebases\dtz6`.
-    #' It is recommended to store .rtbw files on an SSD. There is no loss in
-    #' storing the .rtbz files on a regular HD. It is recommended to verify all
-    #' md5 checksums of the downloaded tablebase files (`md5sum -c checksum.md5`)
-    #' as corruption will lead to engine crashes.
-    #' - `SyzygyProbeDepth`: minimum remaining search depth for which a position
-    #' is probed. Set this option to a higher value to probe less aggressively
-    #' if you experience too much slowdown (in terms of nps) due to TB probing.
-    #' - `Syzygy50MoveRule`: disable to let fifty-move rule draws detected by
-    #' Syzygy tablebase probes count as wins or losses. This is useful for ICCF
-    #' correspondence games.
-    #' - `SyzygyProbeLimit`: limit Syzygy tablebase probing to positions with at
-    #' most this many pieces left (including kings and pawns).
-    #' - `Contempt`: a positive value for contempt favors middle game positions
-    #' and avoids draws, effective for the classical evaluation only.
-    #' - `Analysis Contempt`: by default, contempt is set to prefer the side to
-    #' move. Set this option to "White" or "Black" to analyse with contempt for
-    #' that side, or "Off" to disable contempt.
-    #' - `Move Overhead`: assume a time delay of x ms due to network and GUI
-    #' overheads. This is useful to avoid losses on time in those cases.
-    #' - `Slow Mover`: lower values will make Stockfish take less time in games,
-    #' higher values will make it think longer.
-    #' - `nodestime`: tells the engine to use nodes searched instead of wall time
-    #' to account for elapsed time. Useful for engine testing.
-    #' - `Clear Hash`: clear the hash table.
-    #' - `Debug Log File`: write all communication to and from the engine into a
-    #' text file.
+    #' * `Threads`: The number of CPU threads used for searching a position. For
+    #'   best performance, set this equal to the number of CPU cores available.
+    #'
+    #' * `Hash`: The size of the hash table in MB. It is recommended to set Hash
+    #'   after setting Threads.
+    #'
+    #' * `Clear Hash`: Clear the hash table.
+    #'
+    #' * `Ponder`: Let Stockfish ponder its next move while the opponent is
+    #'   thinking.
+    #'
+    #' * `MultiPV`: Output the N best lines (principal variations, PVs) when
+    #'   searching. Leave at 1 for best performance.
+    #'
+    #' * `Use NNUE`*: Toggle between the NNUE and classical evaluation
+    #'   functions. If set to "true", the network parameters must be available
+    #'   to load from file (see also EvalFile), if they are not embedded in the
+    #'   binary.
+    #'
+    #' * `EvalFile`*: The name of the file of the NNUE evaluation parameters.
+    #'   Depending on the GUI the filename might have to include the full path
+    #'   to the folder/directory that contains the file. Other locations, such
+    #'   as the directory that contains the binary and the working directory,
+    #'   are also searched.
+    #'
+    #' * `UCI_AnalyseMode`: An option handled by your GUI.
+    #'
+    #' * `UCI_Chess960`: An option handled by your GUI. If true, Stockfish will
+    #'   play Chess960.
+    #'
+    #' * `UCI_ShowWDL`: If enabled, show approximate WDL statistics as part of
+    #'   the engine output. These WDL numbers model expected game outcomes for a
+    #'   given evaluation and game ply for engine self-play at fishtest LTC
+    #'   conditions (60+0.6s per game).
+    #'
+    #' * `UCI_LimitStrength`: Enable weaker play aiming for an Elo rating as set
+    #'   by UCI_Elo. This option overrides Skill Level.
+    #'
+    #' * `UCI_Elo`: If enabled by UCI_LimitStrength, aim for an engine strength
+    #'   of the given Elo. This Elo rating has been calibrated at a time control
+    #'   of 60s+0.6s and anchored to CCRL 40/4.
+    #'
+    #' * `Skill Level`: Lower the Skill Level in order to make Stockfish play
+    #'   weaker (see also UCI_LimitStrength). Internally, MultiPV is enabled,
+    #'   and with a certain probability depending on the Skill Level a weaker
+    #'   move will be played.
+    #'
+    #' * `SyzygyPath`: Path to the folders/directories storing the Syzygy
+    #'   tablebase files. Multiple directories are to be separated by ";" on
+    #'   Windows and by ":" on Unix-based operating systems. Do not use spaces
+    #'   around the ";" or ":". Example:
+    #'   `C:\tablebases\wdl345;C:\tablebases\wdl6;D:\tablebases\dtz345;D:\tablebases\dtz6`. #nolint
+    #'   It is recommended to store .rtbw files on an SSD. There is no loss in
+    #'   storing the .rtbz files on a regular HDD. It is recommended to verify
+    #'   all md5 checksums of the downloaded tablebase files
+    #'   (`md5sum -c checksum.md5`) as corruption will lead to engine crashes.
+    #'
+    #' * `SyzygyProbeDepth`: Minimum remaining search depth for which a position
+    #'   is probed. Set this option to a higher value to probe less aggressively
+    #'   if you experience too much slowdown (in terms of nps) due to tablebase
+    #'   probing.
+    #'
+    #' * `Syzygy50MoveRule`: Disable to let fifty-move rule draws detected by
+    #'   Syzygy tablebase probes count as wins or losses. This is useful for
+    #'   ICCF correspondence games.
+    #'
+    #' * `SyzygyProbeLimit`: Limit Syzygy tablebase probing to positions with at
+    #'   most this many pieces left (including kings and pawns).
+    #'
+    #' * `Move Overhead`: Assume a time delay of x ms due to network and GUI
+    #'   overheads. This is useful to avoid losses on time in those cases.
+    #'
+    #' * `Slow Mover`: Lower values will make Stockfish take less time in games,
+    #'   higher values will make it think longer.
+    #'
+    #' * `nodestime`: Tells the engine to use nodes searched instead of wall
+    #'   time to account for elapsed time. Useful for engine testing.
+    #'
+    #' * `Debug Log File`: Write all communication to and from the engine into a
+    #'   text file.
     #'
     #' "This is sent to the engine when the user wants to change the internal
     #' parameters of the engine. For the 'button' type no value is needed. One
@@ -281,11 +307,15 @@ fish <- R6::R6Class(
     #' parsing, for example do not use = 'draw value'. Here are some strings
     #' for the example below:
     #'
-    #' - `setoption name Nullmove value true\n`
-    #' - `setoption name Selectivity value 3\n`
-    #' - `setoption name Style value Risky\n`
-    #' - `setoption name Clear Hash\n`
-    #' - `setoption name NalimovPath value c:\chess\tb\4;c:\chess\tb\5\n`"
+    #' * `setoption name Nullmove value true\n`
+    #'
+    #' * `setoption name Selectivity value 3\n`
+    #'
+    #' * `setoption name Style value Risky\n`
+    #'
+    #' * `setoption name Clear Hash\n`
+    #'
+    #' * `setoption name NalimovPath value c:\chess\tb\4;c:\chess\tb\5\n`"
     #'
     #' @param name Name of the option
     #' @param value Value to set (or `NULL` if option doesn't need a value)
@@ -322,12 +352,12 @@ fish <- R6::R6Class(
     #' sequence of moves, use long algebraic notation (LAN) as described in the
     #' UCI Protocol section of this documentation.
     #'
-    #' "Set up the position described in fenstring on the internal board and play
-    #' the moves on the internal chess board. if the game was played  from the
-    #' start position the string 'startpos' will be sent. Note: no 'new' command
-    #' is needed. However, if this position is from a different game than the
-    #' last position sent to the engine, the GUI should have sent a 'ucinewgame'
-    #' inbetween."
+    #' "Set up the position described in fenstring on the internal board and
+    #' play the moves on the internal chess board. if the game was played from
+    #' the start position the string 'startpos' will be sent. Note: no 'new'
+    #' command is needed. However, if this position is from a different game
+    #' than the last position sent to the engine, the GUI should have sent a
+    #' 'ucinewgame' inbetween."
     #'
     #' @param position String with position (either a FEN or a sequence of moves
     #' separated by spaces)
@@ -344,31 +374,41 @@ fish <- R6::R6Class(
     #' will be sent in the same string. If one command is not send its value
     #' should be interpreted as it would not influence the search.
     #'
-    #' - `searchmoves`: restrict search to this moves only. Example: after
-    #' 'position startpos' and 'go infinite searchmoves e2e4 d2d4' the engine
-    #' should only search the two moves e2e4 and d2d4 in the initial position.
-    #' - `ponder`: start searching in pondering mode. Do not exit the search in
-    #' ponder mode, even if it's mate! This means that the last move sent in the
-    #' position string is the ponder move. The engine can do what it wants to
-    #' do, but after a 'ponderhit' command it should execute the suggested move
-    #' to ponder on. This means that the ponder move sent by the GUI can be
-    #' interpreted as a recommendation about which move to ponder. However, if
-    #' the engine decides to ponder on a different move, it should not display
-    #' any mainlines as they are likely to be misinterpreted by the GUI because
-    #' the GUI expects the engine to ponder on the suggested move.
-    #' - `wtime`: white has x msec left on the clock.
-    #' - `btime`: black has x msec left on the clock.
-    #' - `winc`: white increment per move in mseconds if x > 0.
-    #' - `binc`: black increment per move in mseconds if x > 0.
-    #' - `movestogo`: there are x moves to the next time control, this will only
-    #' be sent if x > 0, if you don't get this and get the wtime and btime it's
-    #' sudden death.
-    #' - `depth`: search x plies only.
-    #' - `nodes`: search x nodes only.
-    #' - `mate`: search for a mate in x moves.
-    #' - `movetime`: search exactly x mseconds.
-    #' - `infinite`: search until the 'stop' command. Do not exit the search
-    #' without being told so in this mode!"
+    #' * `searchmoves`: restrict search to this moves only. Example: after
+    #'   'position startpos' and 'go infinite searchmoves e2e4 d2d4' the engine
+    #'   should only search the two moves e2e4 and d2d4 in the initial position.
+    #'
+    #' * `ponder`: start searching in pondering mode. Do not exit the search in
+    #'   ponder mode, even if it's mate! This means that the last move sent in
+    #'   the position string is the ponder move. The engine can do what it wants
+    #'   to do, but after a 'ponderhit' command it should execute the suggested
+    #'   move to ponder on. This means that the ponder move sent by the GUI can
+    #'   be interpreted as a recommendation about which move to ponder. However,
+    #'   if the engine decides to ponder on a different move, it should not
+    #'   display any mainlines as they are likely to be misinterpreted by the
+    #'   GUI because the GUI expects the engine to ponder on the suggested move.
+    #'
+    #' * `wtime`: white has x msec left on the clock.
+    #'
+    #' * `btime`: black has x msec left on the clock.
+    #'
+    #' * `winc`: white increment per move in mseconds if x > 0.
+    #'
+    #' * `binc`: black increment per move in mseconds if x > 0.
+    #'
+    #' * `movestogo`: there are x moves to the next time control, this will only
+    #'   be sent if x > 0, if you don't get this and get the wtime and btime
+    #'   it's sudden death.
+    #' * `depth`: search x plies only.
+    #'
+    #' * `nodes`: search x nodes only.
+    #'
+    #' * `mate`: search for a mate in x moves.
+    #'
+    #' * `movetime`: search exactly x mseconds.
+    #'
+    #' * `infinite`: search until the 'stop' command. Do not exit the search
+    #'   without being told so in this mode!"
     #'
     #' @param searchmoves A string with the only moves (separated by spaces)
     #' that should be searched
@@ -387,10 +427,12 @@ fish <- R6::R6Class(
     #' @param infinite Whether to only stop searching when a `stop()` command is
     #' issued (makes function return instantly without any output)
     #'
-    #' @return A string with result of the search or `NULL` if `infinite == TRUE`
+    #' @return A string with result of the search or `NULL` if
+    #' `infinite == TRUE`
     go = function(searchmoves = NULL, ponder = NULL, wtime = NULL, btime = NULL,
                   winc = NULL, binc = NULL, movestogo = NULL, depth = NULL,
-                  nodes = NULL, mate = NULL, movetime = NULL, infinite = FALSE) {
+                  nodes = NULL, mate = NULL, movetime = NULL,
+                  infinite = FALSE) {
 
       # List all arguments
       args <- list(
@@ -469,7 +511,7 @@ fish <- R6::R6Class(
     finalize = function() {
       tryCatch(
         self$run("quit"),
-        error = function(err) { }
+        error = function(err) { } #nolint
       )
     }
   )
